@@ -142,6 +142,7 @@ static void magnifier_init_window (Magnifier *magnifier, GdkScreen *screen);
 static gboolean magnifier_check_set_struts (Magnifier *magnifier);
 static gboolean magnifier_reset_struts_at_idle (gpointer data);
 static void magnifier_init_window (Magnifier *magnifier, GdkScreen *screen);
+static void magnifier_adjust_source_size (Magnifier *magnifier);
 static gboolean _is_override_redirect = FALSE;
 
 static Window*
@@ -671,6 +672,45 @@ magnifier_get_display_rect_bounds (Magnifier *magnifier, GNOME_Magnifier_RectBou
     }
 }
 
+static void magnifier_adjust_source_size (Magnifier *magnifier)
+{
+	GNOME_Magnifier_RectBounds rect_bounds;	
+	magnifier_get_display_rect_bounds (magnifier, &rect_bounds, FALSE);
+	gdouble vfract, hfract;
+	hfract = (double) (magnifier->target_bounds.x2 - magnifier->target_bounds.x1) / (double) (rect_bounds.x2 - rect_bounds.x1);
+	vfract = (double) (magnifier->target_bounds.y2 - magnifier->target_bounds.y1) / (double) (rect_bounds.y2 - rect_bounds.y1);
+	if (vfract > hfract) /* vertical splitpane, approximately */
+	{
+		if (magnifier->target_bounds.x1 < magnifier->source_bounds.x1) {
+			magnifier->source_bounds.x1 = magnifier->target_bounds.x2;
+			magnifier->source_bounds.x2 = rect_bounds.x2;
+		}
+		else 
+		{
+			magnifier->source_bounds.x1 = 0;
+			magnifier->source_bounds.x2 = magnifier->target_bounds.x1;
+		}
+		magnifier->source_bounds.y1 = 0;
+		magnifier->source_bounds.y2 = rect_bounds.y2;
+	}
+	else
+	{
+		if (magnifier->target_bounds.y1 < magnifier->source_bounds.y1) {
+			magnifier->source_bounds.y1 = magnifier->target_bounds.y2;
+			magnifier->source_bounds.y2 = rect_bounds.y2;
+		}
+		else 
+		{
+			magnifier->source_bounds.y1 = 0;
+			magnifier->source_bounds.y2 = magnifier->target_bounds.y1;
+		}
+		magnifier->source_bounds.x1 = 0;
+		magnifier->source_bounds.x2 = rect_bounds.x2;
+	}
+	g_message ("set source bounds to %d,%d; %d,%d", 
+		   magnifier->source_bounds.x1, magnifier->source_bounds.y1, magnifier->source_bounds.x2, magnifier->source_bounds.y2);
+}
+
 static void
 magnifier_unref_zoom_region (gpointer data, gpointer user_data)
 {
@@ -1321,6 +1361,9 @@ magnifier_set_property (BonoboPropertyBag *bag,
 		magnifier_check_set_struts (magnifier);
 		DBG(fprintf (stderr, "Set target size: \t%d,%d to %d,%d\n", 
 			      magnifier->target_bounds.x1, magnifier->target_bounds.y1, magnifier->target_bounds.x2, magnifier->target_bounds.y2));
+		if (!strcmp (magnifier->target_display_name, magnifier->source_display_name) && 
+		    (magnifier->target_screen_num == magnifier->source_screen_num)) 
+		    magnifier_adjust_source_size (magnifier);
 		break;
 	case MAGNIFIER_CURSOR_SET_PROP:
 		magnifier_init_cursor_set (magnifier, g_strdup (BONOBO_ARG_GET_STRING (arg)));
